#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date

from dateutil.easter import EASTER_ORTHODOX, EASTER_WESTERN, easter

from holidays.calendars.ethiopian import ETHIOPIAN_CALENDAR, is_ethiopian_leap_year
from holidays.calendars.gregorian import GREGORIAN_CALENDAR, JAN, AUG, SEP, DEC, _timedelta
from holidays.calendars.julian import JULIAN_CALENDAR, julian_calendar_drift
from holidays.calendars.julian_revised import JULIAN_REVISED_CALENDAR


class ChristianHolidays:
    """
    Christian holidays.
    """

    def __init__(self, calendar=GREGORIAN_CALENDAR) -> None:
        self.__verify_calendar(calendar)
        self.__calendar = calendar

    def __get_christmas_day(self, calendar=None):
        """
        Get Christmas Day date.
        """
        calendar = calendar or self.__calendar
        self.__verify_calendar(calendar)

        return (
            _timedelta(date(self._year, JAN, 7), julian_calendar_drift(self._year - 1))
            if self.__is_julian_calendar(calendar) or self.__is_ethiopian_calendar(calendar)
            else date(self._year, DEC, 25)
        )

    def __get_easter_sunday(self, calendar=None):
        """
        Get Easter Sunday date.
        """
        calendar = calendar or self.__calendar
        self.__verify_calendar(calendar)

        return easter(
            self._year,
            method=EASTER_WESTERN if self.__is_gregorian_calendar(calendar) else EASTER_ORTHODOX,
        )

    @staticmethod
    def __is_ethiopian_calendar(calendar):
        """
        Return True if `calendar` is Ethiopian calendar.
        Return False otherwise.
        """
        return calendar == ETHIOPIAN_CALENDAR

    @staticmethod
    def __is_gregorian_calendar(calendar):
        """
        Return True if `calendar` is Gregorian calendar.
        Return False otherwise.
        """
        return calendar == GREGORIAN_CALENDAR

    @staticmethod
    def __is_julian_calendar(calendar):
        """
        Return True if `calendar` is Julian calendar.
        Return False otherwise.
        """
        return calendar == JULIAN_CALENDAR

    @staticmethod
    def __verify_calendar(calendar):
        """
        Verify calendar type.
        """
        if calendar not in {
            ETHIOPIAN_CALENDAR,
            GREGORIAN_CALENDAR,
            JULIAN_CALENDAR,
            JULIAN_REVISED_CALENDAR,
        }:
            raise ValueError(
                f"Unknown calendar name: {calendar}. Use `{ETHIOPIAN_CALENDAR}`, "
                f"`{GREGORIAN_CALENDAR}`, `{JULIAN_CALENDAR}` or `{JULIAN_REVISED_CALENDAR}`."
            )

    @property
    def _christmas_day(self):
        """
        Return Christmas Day date.
        """
        return self.__get_christmas_day()

    @property
    def _easter_sunday(self):
        """
        Return Easter Sunday date.
        """
        return self.__get_easter_sunday()

    def _add_all_saints_day(self, name) -> date:
        """
        Add All Saints' Day (November 1st).

        Also known as All Hallows' Day, the Feast of All Saints,
        the Feast of All Hallows, the Solemnity of All Saints, and Hallowmas.
        https://en.wikipedia.org/wiki/All_Saints'_Day
        """
        return self._add_holiday_nov_1(name)

    def _add_all_souls_day(self, name) -> date:
        """
        Add All Souls' Day (November 2nd).

        All Souls' Day is a day of prayer and remembrance for the faithful
        departed, observed by certain Christian denominations on 2 November.
        In Belarussian tradition it is called Dziady.
        https://en.wikipedia.org/wiki/All_Souls'_Day
        https://en.wikipedia.org/wiki/Dziady
        """
        return self._add_holiday_nov_2(name)

    def _add_ascension_thursday(self, name, calendar=None) -> date:
        """
        Add Ascension Thursday (39 days after the Easter Sunday).

        The Solemnity of the Ascension of Jesus Christ, also called Ascension
        Day, or sometimes Holy Thursday.
        https://en.wikipedia.org/wiki/Feast_of_the_Ascension
        """
        return self._add_holiday(name, _timedelta(self.__get_easter_sunday(calendar), +39))

    def _add_ash_monday(self, name) -> date:
        """
        Add Ash Monday (48 days before Easter Sunday).

        The Clean Monday, also known as Pure Monday, Monday of Lent
        or Green Monday. The first day of Great Lent.
        https://en.wikipedia.org/wiki/Clean_Monday
        """
        return self._add_holiday(name, _timedelta(self._easter_sunday, -48))

    def _add_ash_wednesday(self, name) -> date:
        """
        Add Ash Wednesday (46 days before Easter Sunday).

        A holy day of prayer and fasting. It marks the beginning of Lent.
        https://en.wikipedia.org/wiki/Ash_Wednesday
        """
        return self._add_holiday(name, _timedelta(self._easter_sunday, -46))

    def _add_assumption_of_mary_day(self, name, calendar=None) -> date:
        """
        Add Assumption Of Mary (August 15th).

        The Feast of the Assumption of Mary, or simply The Assumption marks the
        occasion of the Virgin Mary's bodily ascent to heaven at the end of
        her life.
        https://en.wikipedia.org/wiki/Assumption_of_Mary
        """
        calendar = calendar or self.__calendar
        self.__verify_calendar(calendar)

        return (
            self._add_holiday(
                name, _timedelta(date(self._year, AUG, 28), julian_calendar_drift(self._year))
            )
            if self.__is_julian_calendar(calendar)
            else self._add_holiday_aug_15(name)
        )

    def _add_candlemas(self, name) -> date:
        """
        Add Candlemas (February 2nd).

        Also known as the Feast of the Presentation of Jesus Christ,
        the Feast of the Purification of the Blessed Virgin Mary, or the Feast
        of the Holy Encounter, is a Christian holiday commemorating the
        presentation of Jesus at the Temple.
        https://en.wikipedia.org/wiki/Candlemas
        """
        return self._add_holiday_feb_2(name)

    def _add_carnival_sunday(self, name) -> date:
        """
        Add Carnival Sunday (49 days before Easter Sunday).

        Carnival is a Catholic Christian festive season that occurs before
        the liturgical season of Lent.
        https://en.wikipedia.org/wiki/Carnival
        """
        return self._add_holiday(name, _timedelta(self._easter_sunday, -49))

    def _add_carnival_monday(self, name) -> date:
        """
        Add Carnival Monday (48 days before Easter Sunday).

        Carnival is a Catholic Christian festive season that occurs before
        the liturgical season of Lent.
        https://en.wikipedia.org/wiki/Carnival
        """
        return self._add_holiday(name, _timedelta(self._easter_sunday, -48))

    def _add_carnival_tuesday(self, name) -> date:
        """
        Add Carnival Tuesday (47 days before Easter Sunday).

        Carnival is a Catholic Christian festive season that occurs before
        the liturgical season of Lent.
        https://en.wikipedia.org/wiki/Carnival
        """
        return self._add_holiday(name, _timedelta(self._easter_sunday, -47))

    def _add_christmas_day(self, name, calendar=None) -> date:
        """
        Add Christmas Day.

        Christmas is an annual festival commemorating the birth of
        Jesus Christ.
        https://en.wikipedia.org/wiki/Christmas
        """
        return self._add_holiday(name, self.__get_christmas_day(calendar))

    def _add_christmas_day_two(self, name, calendar=None) -> date:
        """
        Add Christmas Day 2.

        A holiday celebrated after Christmas Day, also known as Boxing Day.
        https://en.wikipedia.org/wiki/Boxing_Day
        https://en.wikipedia.org/wiki/Christmas
        """
        return self._add_holiday(name, _timedelta(self.__get_christmas_day(calendar), +1))

    def _add_christmas_day_three(self, name, calendar=None) -> date:
        """
        Add Christmas Day 3.

        A holiday celebrated 2 days after Christmas Day (in some countries).
        https://en.wikipedia.org/wiki/Christmas
        """
        return self._add_holiday(name, _timedelta(self.__get_christmas_day(calendar), +2))

    def _add_christmas_eve(self, name, calendar=None) -> date:
        """
        Add Christmas Eve.

        Christmas Eve is the evening or entire day before Christmas Day,
        the festival commemorating the birth of Jesus Christ.
        https://en.wikipedia.org/wiki/Christmas_Eve
        """
        return self._add_holiday(name, _timedelta(self.__get_christmas_day(calendar), -1))

    def _add_corpus_christi_day(self, name) -> date:
        """
        Add Feast Of Corpus Christi (60 days after Easter Sunday).

        The Feast of Corpus Christi, also known as the Solemnity of the Most
        Holy Body and Blood of Christ, is a Christian liturgical solemnity
        celebrating the Real Presence of the Body and Blood, Soul and Divinity
        of Jesus Christ in the elements of the Eucharist.
        https://en.wikipedia.org/wiki/Feast_of_Corpus_Christi
        """
        return self._add_holiday(name, _timedelta(self._easter_sunday, +60))

    def _add_easter_monday(self, name, calendar=None) -> date:
        """
        Add Easter Monday (1 day after Easter Sunday).

        Easter Monday refers to the day after Easter Sunday in either the
        Eastern or Western Christian traditions. It is a public holiday in
        some countries.
        https://en.wikipedia.org/wiki/Easter_Monday
        """
        return self._add_holiday(name, _timedelta(self.__get_easter_sunday(calendar), +1))

    def _add_easter_sunday(self, name, calendar=None) -> date:
        """
        Add Easter Sunday.

        Easter, also called Pascha or Resurrection Sunday is a Christian
        festival and cultural holiday commemorating the resurrection of Jesus
        from the dead.
        https://en.wikipedia.org/wiki/Easter
        """
        return self._add_holiday(name, self.__get_easter_sunday(calendar))

    def _add_easter_tuesday(self, name, calendar=None) -> date:
        """
        Add Easter Tuesday (2 day after Easter Sunday).

        Easter Tuesday is the third day of Eastertide and is a holiday in some areas.
        https://en.wikipedia.org/wiki/Easter_Tuesday
        """
        return self._add_holiday(name, _timedelta(self.__get_easter_sunday(calendar), +2))

    def _add_epiphany_day(self, name, calendar=None) -> date:
        """
        Add Epiphany Day.

        Epiphany, also known as Theophany in Eastern Christian traditions,
        is a Christian feast day that celebrates the revelation of God
        incarnate as Jesus Christ.
        https://en.wikipedia.org/wiki/Epiphany_(holiday)
        """
        calendar = calendar or self.__calendar
        self.__verify_calendar(calendar)

        if self.__is_julian_calendar(calendar) or self.__is_ethiopian_calendar(calendar):
            dt = _timedelta(date(self._year, JAN, 19), julian_calendar_drift(self._year - 1))
            return self._add_holiday(
                name,
                _timedelta(dt, +1)
                if self.__is_ethiopian_calendar(calendar)
                and is_ethiopian_leap_year(self._year - 1)
                else dt,
            )
        else:
            return self._add_holiday_jan_6(name)

    def _add_finding_of_true_cross(self, name) -> date:
        """
        Add Finding of True Cross.

        Finding of True Cross, also known as Meskel, is an Ethiopian and Eritrean Orthodox
        Tewahedo Church holiday that commemorates the discovery of the True Cross by the
        Roman Empress Saint Helena of Constantinople in the fourth century.
        https://en.wikipedia.org/wiki/Meskel
        """
        dt = _timedelta(date(self._year, SEP, 27), julian_calendar_drift(self._year))
        return self._add_holiday(
            name, _timedelta(dt, +1) if is_ethiopian_leap_year(self._year) else dt
        )

    def _add_good_friday(self, name, calendar=None) -> date:
        """
        Add Good Friday (2 days before Easter Sunday).

        Good Friday is a Christian holiday commemorating the crucifixion of
        Jesus and his death at Calvary. It is also known as Holy Friday,
        Great Friday, Great and Holy Friday.
        https://en.wikipedia.org/wiki/Good_Friday
        """
        return self._add_holiday(name, _timedelta(self.__get_easter_sunday(calendar), -2))

    def _add_holy_saturday(self, name, calendar=None) -> date:
        """
        Add Holy Saturday (1 day before Easter Sunday).

        Great and Holy Saturday is a day between Good Friday and Easter Sunday.
        https://en.wikipedia.org/wiki/Holy_Saturday
        """
        return self._add_holiday(name, _timedelta(self.__get_easter_sunday(calendar), -1))

    def _add_holy_thursday(self, name, calendar=None) -> date:
        """
        Add Holy Thursday (3 days before Easter Sunday).

        Holy Thursday or Maundy Thursday is the day during Holy Week that
        commemorates the Washing of the Feet (Maundy) and Last Supper of
        Jesus Christ with the Apostles, as described in the canonical gospels.
        https://en.wikipedia.org/wiki/Maundy_Thursday
        """
        return self._add_holiday(name, _timedelta(self.__get_easter_sunday(calendar), -3))

    def _add_immaculate_conception_day(self, name) -> date:
        """
        Add Immaculate Conception Day (December 8th).

        https://en.wikipedia.org/wiki/Immaculate_Conception
        """
        return self._add_holiday_dec_8(name)

    def _add_nativity_of_mary_day(self, name) -> date:
        """
        Add Nativity Of Mary Day (September 8th).

        The Nativity of the Blessed Virgin Mary, the Nativity of Mary,
        the Marymas or the Birth of the Virgin Mary, refers to a Christian
        feast day celebrating the birth of Mary, mother of Jesus.
        https://en.wikipedia.org/wiki/Nativity_of_Mary
        """
        return self._add_holiday_sep_8(name)

    def _add_palm_sunday(self, name, calendar=None) -> date:
        """
        Add Palm Sunday (7 days before Easter Sunday).

        Palm Sunday is a Christian moveable feast that falls on the Sunday
        before Easter. The feast commemorates Christ's triumphal entry into
        Jerusalem, an event mentioned in each of the four canonical Gospels.
        Palm Sunday marks the first day of Holy Week.
        https://en.wikipedia.org/wiki/Palm_Sunday
        """
        return self._add_holiday(name, _timedelta(self.__get_easter_sunday(calendar), -7))

    def _add_rejoicing_day(self, name) -> date:
        """
        Add Day Of Rejoicing (9 days after Easter Sunday).

        Add Day Of Rejoicing ("Radonitsa"), in the Russian Orthodox Church is
        a commemoration of the departed observed on the second Tuesday of
        Pascha (Easter). In Ukrainian tradition it is called Provody.
        https://en.wikipedia.org/wiki/Radonitsa
        """
        return self._add_holiday(name, _timedelta(self._easter_sunday, +9))

    def _add_saint_anthonys_day(self, name) -> date:
        """
        Add Saint Anthony of Padua's Day (June 13th).

        Saint Anthony's Day is celebrated on 13 June, the traditionally
        accepted date of the saint's death.
        https://en.wikipedia.org/wiki/Anthony_of_Padua
        """
        return self._add_holiday_jun_13(name)

    def _add_saint_georges_day(self, name) -> date:
        """
        Add Saint George's Day (April 23th).

        Saint George's Day is celebrated on 23 April, the traditionally
        accepted date of the saint's death.
        https://en.wikipedia.org/wiki/Saint_George's_Day
        """
        return self._add_holiday_apr_23(name)

    def _add_saint_james_day(self, name) -> date:
        """
        Add Saint James' Day (July 25th).

        James the Great was one of the Twelve Apostles of Jesus.
        https://en.wikipedia.org/wiki/James_the_Great#Feast
        """
        return self._add_holiday_jul_25(name)

    def _add_saint_johns_day(self, name) -> date:
        """
        Add Saint John's Day (June 24th).

        The Nativity of John the Baptist is a Christian feast day celebrating
        the birth of John the Baptist.
        https://en.wikipedia.org/wiki/Nativity_of_John_the_Baptist
        """
        return self._add_holiday_jun_24(name)

    def _add_saint_josephs_day(self, name) -> date:
        """
        Add Saint Joseph's Day (March 19th).

        Saint Joseph's Day, also called the Feast of Saint Joseph or the
        Solemnity of Saint Joseph, is in Western Christianity the principal
        feast day of Saint Joseph, husband of the Virgin Mary and legal father
        of Jesus Christ.
        https://en.wikipedia.org/wiki/Saint_Joseph's_Day
        """
        return self._add_holiday_mar_19(name)

    def _add_saint_martins_day(self, name) -> date:
        """
        Add Saint Martin of Tours Day (November 11th).

        Saint Martin of Tours is the patron saint of many communities and organizations
        across Europe, including France's Third Republic.
        https://en.wikipedia.org/wiki/Martin_of_Tours
        """
        return self._add_holiday_nov_11(name)

    def _add_saint_patricks_day(self, name) -> date:
        """
        Add Saint Patrick's Day (March 17th).

        Saint Patrick's Day is a religious and cultural holiday held on 17 March,
        the traditional death date of Saint Patrick, the foremost patron saint of Ireland.
        https://en.wikipedia.org/wiki/Saint_Patrick's_Day
        """
        return self._add_holiday_mar_17(name)

    def _add_saints_peter_and_paul_day(self, name) -> date:
        """
        Add Feast of Saints Peter and Paul (June 29th).

        A liturgical feast in honor of the martyrdom in Rome of the apostles
        Saint Peter and Saint Paul, which is observed on 29 June.
        https://en.wikipedia.org/wiki/Feast_of_Saints_Peter_and_Paul
        """
        return self._add_holiday_jun_29(name)

    def _add_whit_monday(self, name) -> date:
        """
        Add Whit Monday (50 days after Easter Sunday).

        Whit Monday or Pentecost Monday, also known as Monday of the
        Holy Spirit, is the holiday celebrated the day after Pentecost.
        https://en.wikipedia.org/wiki/Pentecost
        https://en.wikipedia.org/wiki/Whit_Monday
        """
        return self._add_holiday(name, _timedelta(self._easter_sunday, +50))

    def _add_whit_sunday(self, name, calendar=None) -> date:
        """
        Add Whit Sunday (49 days after Easter Sunday).

        Whit Sunday, also called Pentecost, is a holiday which commemorates
        the descent of the Holy Spirit upon the Apostles and other followers
        of Jesus Christ while they were in Jerusalem celebrating the
        Feast of Weeks.
        https://en.wikipedia.org/wiki/Pentecost
        """
        return self._add_holiday(name, _timedelta(self.__get_easter_sunday(calendar), +49))

    def _add_trinity_sunday(self, name) -> date:
        """
        Add Trinity Sunday (56 days after Easter Sunday).

        Trinity Sunday, also called Solemnity of Holy Trinity, is the first Sunday
        after Pentecost in the Western Christian liturgical calendar, and the Sunday
        of Pentecost in Eastern Christianity.
        """
        return self._add_holiday(name, _timedelta(self._easter_sunday, +56))



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import JAN, APR, MAY, JUN, JUL, AUG, SEP, OCT, NOV, DEC, FRI, SAT
from holidays.groups import InternationalHolidays, IslamicHolidays, PersianCalendarHolidays
from holidays.holiday_base import HolidayBase


class Afghanistan(HolidayBase, InternationalHolidays, IslamicHolidays, PersianCalendarHolidays):
    """Afghanistan holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Afghanistan>
        * <https://web.archive.org/web/20250903162748/https://www.timeanddate.com/holidays/afghanistan>
        * <https://en.wikipedia.org/wiki/Workweek_and_weekend>
    """

    country = "AF"
    default_language = "fa_AF"
    # %s (estimated).
    estimated_label = tr("%s (برآورد شده)")
    supported_languages = ("en_US", "fa_AF", "ps_AF")
    # Afghanistan's regaining of full independence from British influence.
    start_year = 1919
    weekend = {FRI, SAT}

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=AfghanistanIslamicHolidays, show_estimated=islamic_show_estimated
        )
        PersianCalendarHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        if self._year >= 1989:
            # Liberation Day.
            self._add_holiday_feb_15(tr("روز آزادی"))

        # Afghanistan Independence Day.
        self._add_holiday_aug_19(tr("روز استقلال افغانستان"))

        if self._year <= 1996 or 2001 <= self._year <= 2020:
            # Nowruz.
            self._add_nowruz_day(tr("نوروز"))

        if self._year >= 1992:
            # Mojahedin's Victory Day.
            self._add_holiday_apr_28(tr("روز پیروزی مجاهدین"))

        if 1974 <= self._year <= 1996 or 2002 <= self._year <= 2021:
            # International Workers' Day.
            self._add_labor_day(tr("روز جهانی کارگر"))

        if 1978 <= self._year <= 1988:
            # Soviet Victory Day.
            self._add_holiday_may_9(tr("روز پیروزی شوروی"))

        if self._year >= 2022:
            # Islamic Emirate Victory Day.
            self._add_islamic_emirat_victory_day(tr("روز پیروزی امارت اسلامی"))

            # American Withdrawal Day.
            self._add_holiday_aug_31(tr("روز خروج آمریکایی ها"))

        if 2012 <= self._year <= 2020:
            # Martyrs' Day.
            self._add_holiday_sep_9(tr("روز شهیدان"))

        if self._year <= 2021:
            # Ashura.
            self._add_ashura_day(tr("عاشورا"))

        # Prophet's Birthday.
        self._add_mawlid_day(tr("میلاد پیامبر"))

        # First Day of Ramadan.
        self._add_ramadan_beginning_day(tr("اول رمضان"))

        # Eid al-Fitr.
        name = tr("عید فطر")
        self._add_eid_al_fitr_day(name)
        self._add_eid_al_fitr_day_two(name)
        self._add_eid_al_fitr_day_three(name)

        # Day of Arafah.
        self._add_arafah_day(tr("روز عرفه"))

        # Eid al-Adha.
        name = tr("عید قربانی")
        self._add_eid_al_adha_day(name)
        self._add_eid_al_adha_day_two(name)
        self._add_eid_al_adha_day_three(name)


class AF(Afghanistan):
    pass


class AFG(Afghanistan):
    pass


class AfghanistanIslamicHolidays(_CustomIslamicHolidays):
    ASHURA_DATES_CONFIRMED_YEARS = (2014, 2021)
    ASHURA_DATES = {
        2015: (OCT, 24),
        2016: (OCT, 12),
        2017: (OCT, 1),
        2018: (SEP, 21),
        2019: (SEP, 10),
        2020: (AUG, 30),
        2021: (AUG, 19),
    }

    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2014, 2025)
    EID_AL_ADHA_DATES = {
        2014: (OCT, 5),
        2016: (SEP, 13),
        2017: (SEP, 2),
        2018: (AUG, 22),
        2024: (JUN, 17),
        2025: (JUN, 7),
    }

    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2014, 2025)
    EID_AL_FITR_DATES = {
        2014: (JUL, 29),
        2015: (JUL, 18),
        2016: (JUL, 7),
        2017: (JUN, 26),
        2022: (MAY, 1),
        2023: (APR, 22),
    }

    MAWLID_DATES_CONFIRMED_YEARS = (2014, 2025)
    MAWLID_DATES = {
        2014: (JAN, 14),
        2015: ((JAN, 3), (DEC, 24)),
        2016: (DEC, 12),
        2017: (DEC, 1),
        2018: (NOV, 21),
        2019: (NOV, 10),
        2021: (OCT, 19),
        2024: (SEP, 16),
        2025: (SEP, 5),
    }

    RAMADAN_BEGINNING_DATES_CONFIRMED_YEARS = (2014, 2025)
    RAMADAN_BEGINNING_DATES = {
        2014: (JUN, 29),
        2016: (JUN, 7),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.calendars.gregorian import APR, JUN, SEP, NOV
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SAT_TO_PREV_FRI,
    SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON_TUE,
)


class SolomonIslands(
    ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays
):
    """Solomon Islands holidays.

    References:
        * [Public Holidays Act](https://web.archive.org/web/20250615054807/https://www.paclii.org/sb/legis/consol_act_1996/pha163.pdf)
        * [2016](https://web.archive.org/web/20250530230724/https://mehrd.gov.sb/documents?view=download&format=raw&fileId=4377)
        * [2020](https://web.archive.org/web/20240908025223/https://solomons.gov.sb/national-and-provincial-public-holidays-2020/)
        * [2021](https://web.archive.org/web/20241004035112/https://solomons.gov.sb/wp-content/uploads/2021/03/Gaz-No.-54-Sup-No.-50-Wednesday-10th-March-2021.pdf)
        * [2022](https://web.archive.org/web/20240727185307/https://solomons.gov.sb/wp-content/uploads/2021/10/Gaz-No.-246-Tuesday-19th-October-2021-.pdf)
        * [2023](https://web.archive.org/web/20240810050725/https://solomons.gov.sb/wp-content/uploads/2022/12/Gaz-No.-324-Wednesday-21st-December-2022-1.pdf)
        * [2024](https://web.archive.org/web/20240727124555/https://solomons.gov.sb/wp-content/uploads/2024/02/Gaz-No.-15-Monday-12th-FEBRUARY-2024.pdf)
        * [2025](https://web.archive.org/web/20250615054517/https://solomons.gov.sb/wp-content/uploads/2024/11/Gaz-No.-178-Monday-25th-November-2024.pdf)

    About the day appointed for the celebration of the Anniversary of the Birthday of the
    Sovereign:

    * Up to 2022, the Queen's Birthday was typically observed on the second Saturday of
        June, with the preceding Friday designated as a public holiday. From 2023 onward,
        the King's Birthday has been marked on the Friday before the third Saturday in
        June. Although there has been no amendment to the Public Holidays Act to explicitly
        state this change, the pattern is evident in the annual government gazettes and
        supported by information from the relevant [Wikipedia
        entry](https://en.wikipedia.org/wiki/King's_Official_Birthday#Solomon_Islands).
    * There are a few exceptions to this rule:
        * [Queen's Birthday 2022](https://web.archive.org/web/20240727184251/https://solomons.gov.sb/wp-content/uploads/2022/05/Gaz-No.-171-Friday-27th-May-2022.pdf)
        * According to the holidays schedule for 2024, King's Birthday was celebrated on
          the 3rd saturday of June (June 15) and the preceding Friday was observed as a
          public holiday.
        * [King's Birthday 2025](https://web.archive.org/web/20250615054519/https://solomons.gov.sb/wp-content/uploads/2025/04/Gaz-No.-33-Wednesday-9th-April-2025.pdf)

    Province Days are not listed in the Public Holidays Act but are consistently announced
    in the official gazette each year.

    While the Public Holidays Act specifies the Sunday to next Monday observance rule,
    gazettes dating back to 2016 also follow the Saturday to previous Friday rule for
    shifting public holidays.
    """

    country = "SB"
    # %s (observed).
    observed_label = "%s (observed)"
    start_year = 1979
    subdivisions = (
        "CE",  # Central.
        "CH",  # Choiseul.
        "CT",  # Capital Territory (Honiara).
        "GU",  # Guadalcanal.
        "IS",  # Isabel.
        "MK",  # Makira-Ulawa.
        "ML",  # Malaita.
        "RB",  # Rennell and Bellona.
        "TE",  # Temotu.
        "WE",  # Western.
    )
    subdivisions_aliases = {
        "Central": "CE",
        "Choiseul": "CH",
        "Capital Territory": "CT",
        "Honiara": "CT",
        "Guadalcanal": "GU",
        "Isabel": "IS",
        "Makira-Ulawa": "MK",
        "Malaita": "ML",
        "Rennell and Bellona": "RB",
        "Temotu": "TE",
        "Western": "WE",
    }

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, cls=SolomonIslandsStaticHolidays)
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON + SAT_TO_PREV_FRI)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        name = "New Year's Day"
        self._add_observed(self._add_new_years_day(name))
        self._add_observed(self._next_year_new_years_day, name=name, rule=SAT_TO_PREV_FRI)

        # Good Friday.
        self._add_good_friday("Good Friday")

        # Holy Saturday.
        self._add_holy_saturday("Holy Saturday")

        # Easter Monday.
        self._add_easter_monday("Easter Monday")

        # Whit Monday.
        self._add_whit_monday("Whit Monday")

        name = (
            # King's Birthday.
            "King's Birthday"
            if self._year >= 2023
            # Queen's Birthday.
            else "Queen's Birthday"
        )
        sovereign_birthday_dates = {
            2022: (JUN, 3),
            2025: (JUN, 13),
        }
        if dt := sovereign_birthday_dates.get(self._year):
            self._add_holiday(name, dt)
        elif self._year == 2024:
            self._add_observed(self._add_holiday_3rd_sat_of_jun(name))
        elif self._year >= 2023:
            self._add_holiday_1_day_prior_3rd_sat_of_jun(name)
        else:
            self._add_observed(self._add_holiday_2nd_sat_of_jun(name))

        # Independence Day.
        self._add_observed(self._add_holiday_jul_7("Independence Day"))

        self._add_observed(
            # Christmas Day.
            self._add_christmas_day("Christmas Day"),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )

        self._add_observed(
            # National Day of Thanksgiving.
            self._add_christmas_day_two("National Day of Thanksgiving"),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )

    def _populate_subdiv_ce_public_holidays(self):
        # Central Province Day.
        self._add_observed(self._add_holiday_jun_29("Central Province Day"))

    def _populate_subdiv_ch_public_holidays(self):
        # Choiseul Province Day.
        self._add_observed(self._add_holiday_feb_25("Choiseul Province Day"))

    def _populate_subdiv_gu_public_holidays(self):
        # Guadalcanal Province Day.
        self._add_observed(self._add_holiday_aug_1("Guadalcanal Province Day"))

    def _populate_subdiv_is_public_holidays(self):
        # Isabel Province Day.
        self._add_observed(self._add_holiday_jun_2("Isabel Province Day"))

    def _populate_subdiv_mk_public_holidays(self):
        # Makira-Ulawa Province Day.
        self._add_observed(self._add_holiday_aug_3("Makira-Ulawa Province Day"))

    def _populate_subdiv_ml_public_holidays(self):
        # Malaita Province Day.
        self._add_observed(self._add_holiday_aug_15("Malaita Province Day"))

    def _populate_subdiv_rb_public_holidays(self):
        # Rennell and Bellona Province Day.
        self._add_observed(self._add_holiday_jul_20("Rennell and Bellona Province Day"))

    def _populate_subdiv_te_public_holidays(self):
        # Temotu Province Day.
        self._add_observed(self._add_holiday_jun_8("Temotu Province Day"))

    def _populate_subdiv_we_public_holidays(self):
        # Western Province Day.
        self._add_observed(self._add_holiday_dec_7("Western Province Day"))


class SB(SolomonIslands):
    pass


class SLB(SolomonIslands):
    pass


class SolomonIslandsStaticHolidays:
    """Solomon Islands special holidays.

    References:
        * [2024 General Election Holiday](https://web.archive.org/web/20240711032116/https://solomonchamber.com.sb/media/2618/gaz-no-61-monday-8th-april-2024.pdf)
        * [2020 By-election Holiday](https://web.archive.org/web/20241011021616/https://solomons.gov.sb/wp-content/uploads/2020/11/GAZ-173-11th-November-2020-1.pdf)
        * [Gazette No. 269 of 2022](https://web.archive.org/web/20240727183925/https://solomons.gov.sb/wp-content/uploads/2022/09/Gaz-No.-269-Friday-9th-September-2022.pdf)
    """

    # Public Holiday.
    name = "Public Holiday"
    special_public_holidays = {
        2022: (SEP, 12, name),
        2024: (APR, 17, name),
    }

    special_ch_public_holidays = {
        2020: (NOV, 18, name),
    }

    special_gu_public_holidays = {
        2020: (NOV, 18, name),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from __future__ import annotations

from gettext import gettext as tr
from typing import TYPE_CHECKING

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import (
    FEB,
    APR,
    MAY,
    JUN,
    JUL,
    AUG,
    SEP,
    OCT,
    NOV,
    DEC,
    FRI,
    SAT,
    SUN,
)
from holidays.constants import GOVERNMENT, OPTIONAL, PUBLIC
from holidays.groups import InternationalHolidays, IslamicHolidays, StaticHolidays
from holidays.holiday_base import HolidayBase

if TYPE_CHECKING:
    from datetime import date


class UnitedArabEmirates(HolidayBase, InternationalHolidays, IslamicHolidays, StaticHolidays):
    """United Arab Emirates holidays.

    References:
    * [2017](https://web.archive.org/web/20230209125034/https://www.khaleejtimes.com/nation/uae-official-public-holidays-list-2017)
    * [2018](https://web.archive.org/web/20201111212817/https://www.khaleejtimes.com/nation/Here-are-the-holidays-remaining-in-2018-in-UAE)
    * [2019](https://web.archive.org/web/20240513133929/https://www.thenational.ae/uae/government/uae-public-holidays-for-2019-and-2020-announced-by-cabinet-1.833425)
    * [2020](https://web.archive.org/web/20250424164934/https://u.ae/en/information-and-services/public-holidays-and-religious-affairs/public-holidays)
    * [2021](https://web.archive.org/web/20210928140451/http://wam.ae/en/details/1395302957696)
    * [2022](https://web.archive.org/web/20250207171255/https://www.khaleejtimes.com/ramadan/eid-al-fitr-holiday-announced-in-uae-3)
    * 2023:
        * <https://web.archive.org/web/20231207191645/https://www.timeoutdubai.com/news/uae-public-holidays-in-2023>
        * <https://web.archive.org/web/20250125141309/https://www.khaleejtimes.com/uae/islamic-new-year-2023-uae-announces-official-holiday-for-public-sector>
    * [2024](https://web.archive.org/web/20241001104607/https://www.timeoutdubai.com/news/uae-public-holidays-in-2024)
    * [2025](https://web.archive.org/web/20250415131512/https://www.timeoutdubai.com/news/uae-public-holidays-2025)

    Holidays are regulated by the Article 74 of [Federal Law No.
    08](https://web.archive.org/web/20250304004437/https://www.ilo.org/dyn/natlex/docs/ELECTRONIC/11956/69376/F417089305/ARE11956.pdf)
    for the year 1980.

    However the law is not applied literally, and was amended often in the past few years.
    """

    country = "AE"
    default_language = "ar"
    # %s (estimated).
    estimated_label = tr("%s (المقدرة)")
    supported_categories = (GOVERNMENT, OPTIONAL, PUBLIC)
    supported_languages = ("ar", "en_US", "th")
    # Founded on DEC 2, 1971.
    start_year = 1972

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=UnitedArabEmiratesIslamicHolidays, show_estimated=islamic_show_estimated
        )
        StaticHolidays.__init__(self, UnitedArabEmiratesStaticHolidays)
        super().__init__(*args, **kwargs)

    def _get_weekend(self, dt: date) -> set[int]:
        # The resting days are Saturday and Sunday since Jan 1, 2022.
        # https://web.archive.org/web/20250216144205/https://time.com/6126260/uae-working-days-weekend/
        return {SAT, SUN} if dt.year >= 2022 else {FRI, SAT}

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("رأس السنة الميلادية"))

        if 2015 <= self._year <= 2023:
            # Commemoration Day.
            name = tr("يوم الشهيد")
            if self._year >= 2019:
                self._add_holiday_dec_1(name)
            else:
                self._add_holiday_nov_30(name)

        # National Day.
        national_day = tr("اليوم الوطني")
        self._add_holiday_dec_2(national_day)
        self._add_holiday_dec_3(national_day)

        # Eid al-Fitr.
        self._add_eid_al_fitr_day(tr("عيد الفطر"))

        # Eid al-Fitr Holiday.
        eid_al_fitr_holiday = tr("عطلة عيد الفطر")
        self._add_eid_al_fitr_day_two(eid_al_fitr_holiday)
        self._add_eid_al_fitr_day_three(eid_al_fitr_holiday)
        if 2019 <= self._year <= 2024:
            # Ramadan 30 not confirmed yet for 2025 onwards.
            self._add_eid_al_fitr_eve(eid_al_fitr_holiday)

        # Arafat Day.
        self._add_arafah_day(tr("وقفة عرفة"))

        # Eid al-Adha.
        self._add_eid_al_adha_day(tr("عيد الأضحى"))

        # Eid al-Adha Holiday.
        eid_al_adha_holiday = tr("عطلة عيد الأضحى")
        self._add_eid_al_adha_day_two(eid_al_adha_holiday)
        self._add_eid_al_adha_day_three(eid_al_adha_holiday)

        # Islamic New Year.
        self._add_islamic_new_year_day(tr("رأس السنة الهجرية"))

        if self._year <= 2018:
            # Isra' and Mi'raj.
            self._add_isra_and_miraj_day(tr("ليلة المعراج"))

        # Prophet's Birthday.
        self._add_mawlid_day(tr("عيد المولد النبوي"))


class AE(UnitedArabEmirates):
    pass


class ARE(UnitedArabEmirates):
    pass


class UnitedArabEmiratesIslamicHolidays(_CustomIslamicHolidays):
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2017, 2025)
    EID_AL_ADHA_DATES = {
        2018: (AUG, 22),
        2025: (JUN, 4),
    }

    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2017, 2025)

    HIJRI_NEW_YEAR_DATES_CONFIRMED_YEARS = (2017, 2025)
    HIJRI_NEW_YEAR_DATES = {
        2017: (SEP, 22),
        2020: (AUG, 23),
        2021: (AUG, 12),
        2023: (JUL, 21),
    }

    ISRA_AND_MIRAJ_DATES_CONFIRMED_YEARS = (2017, 2018)
    ISRA_AND_MIRAJ_DATES = {
        2017: (APR, 23),
        2018: (APR, 14),
    }

    MAWLID_DATES_CONFIRMED_YEARS = (2017, 2025)
    MAWLID_DATES = {
        2018: (NOV, 18),
        2021: (OCT, 21),
        2023: (SEP, 29),
        2025: (SEP, 5),
    }


class UnitedArabEmiratesStaticHolidays:
    """United Arab Emirates special holidays.

    Special Eid al-Fitr entries for Ramadan 29 from 2020 onwards:
        * [2019](https://web.archive.org/web/20250429081745/https://www.mohre.gov.ae/en/media-centre/news/30/1/2019/عطلة-رسمية-يوم-زيارة-البابا-للمشاركين-في-فعالياته-من-القطاع-الخاص.aspx?DisableResponsive=1)
        * [2020](https://web.archive.org/web/20210804023600/https://gulfbusiness.com/revealed-uae-private-sector-holidays-for-eid-al-fitr-2020/)
        * [2021](https://web.archive.org/web/20220508191126/https://www.timeoutdubai.com/news/466278-eid-al-fitr-holiday-2021-dubai)
        * 2022:
            * <https://web.archive.org/web/20241210072332/https://gulfnews.com/uae/eid-al-fitr-2022-holidays-for-private-and-public-sectors-in-uae-explained-1.1650951429432>
            * <https://web.archive.org/web/20220616134156/https://www.arabianbusiness.com/gcc/uae/uae-suspends-work-for-three-days-marks-40-days-of-mourning-over-sheikh-khalifas-death>
        * [2023](https://web.archive.org/web/20240301231911/https://hrme.economictimes.indiatimes.com/news/industry/uae-cabinet-announces-national-day-holiday-for-federal-government-from-2-to-4-december/105455071)
        * [2024](https://web.archive.org/web/20240720085937/http://www.timeoutdubai.com/news/eid-al-fitr-2024-expected-dates-ramadan)
        * [2025](https://web.archive.org/web/20250415131512/https://www.timeoutdubai.com/news/uae-public-holidays-2025)

    Cross-Checked With:
        * <https://web.archive.org/web/20250429081729/https://www.timeanddate.com/holidays/united-arab-emirates/2021?hol=134217729>
    """

    # Eid al-Fitr Holiday.
    eid_al_fitr_holiday = tr("عطلة عيد الفطر")
    # Death of Sheikh Khalifa bin Zayed Al Nahyan.
    sheikh_khalifa_bin_zayed_death = tr("وفاة الشيخ خليفة بن زايد آل نهيان")

    special_government_holidays = {
        2022: (
            (MAY, 5, eid_al_fitr_holiday),
            (MAY, 6, eid_al_fitr_holiday),
            (MAY, 7, eid_al_fitr_holiday),
            (MAY, 8, eid_al_fitr_holiday),
        ),
        # National Day.
        2023: (DEC, 4, tr("اليوم الوطني")),
    }
    special_optional_holidays = {
        # Pope Francis' Visit Day.
        2019: (FEB, 5, tr("يوم زيارة البابا فرنسيس")),
    }
    special_public_holidays = {
        2020: (MAY, 22, eid_al_fitr_holiday),
        2021: (MAY, 11, eid_al_fitr_holiday),
        2022: (
            (APR, 30, eid_al_fitr_holiday),
            (MAY, 14, sheikh_khalifa_bin_zayed_death),
            (MAY, 15, sheikh_khalifa_bin_zayed_death),
            (MAY, 16, sheikh_khalifa_bin_zayed_death),
        ),
        2024: (APR, 8, eid_al_fitr_holiday),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import FRI, SAT
from holidays.groups import InternationalHolidays
from holidays.holiday_base import HolidayBase


class Bangladesh(HolidayBase, InternationalHolidays):
    """Bangladesh holidays.

    References:
        * <https://web.archive.org/web/20241109215908/https://mopa.gov.bd/sites/default/files/files/mopa.gov.bd/public_holiday/61c35b73_e335_462a_9bcf_4695b23b6d82/reg4-2019-212.PDF>
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Bangladesh>
    """

    country = "BD"
    default_language = "bn"
    supported_languages = ("bn", "en_US")
    weekend = {FRI, SAT}

    def __init__(self, *args, **kwargs):
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # Martyrs' Day and International Mother Language Day.
        self._add_holiday_feb_21(tr("শহীদ দিবস ও আন্তর্জাতিক মাতৃভাষা দিবস"))

        # Sheikh Mujibur Rahman's Birthday.
        self._add_holiday_mar_17(tr("জাতির পিতা বঙ্গবন্ধু শেখ মুজিবুর রহমান এর জন্মদিবস"))

        # Independence Day.
        self._add_holiday_mar_26(tr("স্বাধীনতা দিবস"))

        # Bengali New Year's Day.
        self._add_holiday_apr_14(tr("পহেলা বৈশাখ"))

        # May Day.
        self._add_labor_day(tr("মে দিবস"))

        # National Mourning Day.
        self._add_holiday_aug_15(tr("জাতীয় শোক দিবস"))

        # Victory Day.
        self._add_holiday_dec_16(tr("বিজয় দিবস"))


class BD(Bangladesh):
    pass


class BGD(Bangladesh):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class Mexico(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Mexico holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Mexico>
        * <https://es.wikipedia.org/wiki/Anexo:Días_festivos_en_México>
        * <https://web.archive.org/web/20250421111013/https://www.gob.mx/cms/uploads/attachment/file/156203/1044_Ley_Federal_del_Trabajo.pdf>
        * <https://web.archive.org/web/20250427131751/http://www.diputados.gob.mx/LeyesBiblio/ref/lft/LFT_orig_01abr70_ima.pdf>
        * <https://web.archive.org/web/20250414071145/https://www.gob.mx/profedet/es/articulos/sabes-cuales-son-los-dias-de-descanso-obligatorio-para-este-2024>
    """

    country = "MX"
    default_language = "es"
    supported_languages = ("en_US", "es", "uk")

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Año Nuevo"))

        if self._year >= 1917:
            # Constitution Day.
            name = tr("Día de la Constitución")
            if self._year >= 2006:
                self._add_holiday_1st_mon_of_feb(name)
            else:
                self._add_holiday_feb_5(name)

        if self._year >= 1917:
            # Benito Juárez's birthday.
            name = tr("Natalicio de Benito Juárez")
            # no 2006 due to celebration of the 200th anniversary
            # of Benito Juárez in 2006
            if self._year >= 2007:
                self._add_holiday_3rd_mon_of_mar(name)
            else:
                self._add_holiday_mar_21(name)

        if self._year >= 1923:
            # Labor Day.
            self._add_labor_day(tr("Día del Trabajo"))

        # Independence Day.
        self._add_holiday_sep_16(tr("Día de la Independencia"))

        if self._year >= 1917:
            # Revolution Day.
            name = tr("Día de la Revolución")
            if self._year >= 2006:
                self._add_holiday_3rd_mon_of_nov(name)
            else:
                self._add_holiday_nov_20(name)

        if self._year >= 1970 and (self._year - 1970) % 6 == 0:
            # Change of Federal Government.
            name = tr("Transmisión del Poder Ejecutivo Federal")
            if self._year >= 2024:
                self._add_holiday_oct_1(name)
            else:
                self._add_holiday_dec_1(name)

        # Christmas Day.
        self._add_christmas_day(tr("Navidad"))


class MX(Mexico):
    pass


class MEX(Mexico):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date

from holidays.calendars.balinese_saka import _BalineseSakaLunar


class BalineseSakaCalendarHolidays:
    """
    Balinese Saka lunar calendar holidays.
    """

    def __init__(self) -> None:
        self._balinese_saka_calendar = _BalineseSakaLunar()

    def _add_balinese_saka_calendar_holiday(self, name: str, dt: date | None) -> date | None:
        """
        Add Balinese Saka calendar holiday.
        """
        if dt is None:
            return None
        return self._add_holiday(name, dt)

    def _add_nyepi(self, name) -> date | None:
        """
        Add Nyepi (Day following the 9th of Dark Moon (Tilem)).

        Nyepi is a Balinese "Day of Silence" that is commemorated every
        Isakawarsa (Saka new year) according to the Balinese calendar.
        https://en.wikipedia.org/wiki/Nyepi
        """
        return self._add_balinese_saka_calendar_holiday(
            name, self._balinese_saka_calendar.nyepi_date(self._year)
        )
